package com.qs.mp.web.controller.api.admin;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.qs.mp.admin.domain.CdKey;
import com.qs.mp.admin.domain.CdKeyGroup;
import com.qs.mp.admin.domain.CdKeyGroupGoods;
import com.qs.mp.admin.domain.excel.CdKeyOrderExcel;
import com.qs.mp.admin.domain.param.CdKeyActivateParam;
import com.qs.mp.admin.domain.param.CdKeyCreateParam;
import com.qs.mp.admin.domain.param.CdKeyExportParam;
import com.qs.mp.admin.domain.param.CdKeyQueryParam;
import com.qs.mp.admin.domain.vo.CdKeyExchangeVO;
import com.qs.mp.admin.domain.vo.CdKeyGroupListVO;
import com.qs.mp.admin.domain.vo.CdKeyGroupVO;
import com.qs.mp.admin.service.ICdKeyGroupGoodsService;
import com.qs.mp.admin.service.ICdKeyGroupService;
import com.qs.mp.admin.service.ICdKeyService;
import com.qs.mp.channel.domain.Channel;
import com.qs.mp.channel.service.IChannelService;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.core.page.TableDataInfo;
import com.qs.mp.common.enums.CdKeyGroupStatusEnum;
import com.qs.mp.common.enums.CdKeyStatusEnum;
import com.qs.mp.common.utils.StringUtils;
import com.qs.mp.common.utils.bean.BeanUtils;
import com.qs.mp.utils.ExcelUtil;
import com.qs.mp.web.controller.BaseController;
import io.swagger.annotations.*;
import ma.glasnost.orika.MapperFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhangkaikai
 * @create 2023-05-16 10:16 AM
 **/
@Api(tags = "兑换码管理API")
@RestController
@RequestMapping("/api/v1/mp/admin/CdKey/*")
public class CdKeyMgrController extends BaseController {


    @Autowired
    private ICdKeyGroupService cdKeyGroupService;

    @Autowired
    private ICdKeyGroupGoodsService cdKeyGroupGoodsService;

    @Autowired
    private ICdKeyService cdKeyService;

    @Autowired
    private MapperFacade mapperFacade;

    @Autowired
    private IChannelService channelService;

    @Value("${cloud.public-bucket-name}")
    private String publicBucketName;

    @Value("${cloud.region}")
    private String region;


    @ApiOperation("兑换码生成")
    @PreAuthorize("@ss.hasPermi('business:cdKey:add')")
    @PostMapping("/create")
    public AjaxResult create(@Validated @RequestBody CdKeyCreateParam param) {
        String groupId = cdKeyGroupService.createCdKeyGroup(param);
        if (StringUtils.isNotBlank(groupId)) {
            return AjaxResult.success();
        }
        return AjaxResult.error("生成兑换码失败");
    }

    @ApiOperation("查询所有兑换码组列表")
    @PreAuthorize("@ss.hasPermi('business:cdKey:list')")
    @PostMapping("/list")
    @ApiResponses(
            @ApiResponse(code = 200, message = "查询成功", response = CdKeyGroupListVO.class)
    )
    public TableDataInfo list(@RequestBody CdKeyQueryParam param) {

        QueryWrapper<CdKeyGroup> queryWrapper = new QueryWrapper<>();

        if (Objects.nonNull(param.getCreateTimeStart()) && Objects.nonNull(param.getCreateTimeEnd())) {
            queryWrapper.between("created_time", param.getCreateTimeStart(), param.getCreateTimeEnd());
        }

        if (Objects.nonNull(param.getActiveTimeStart()) && Objects.nonNull(param.getActiveTimeEnd())) {
            queryWrapper.between("activation_time", param.getActiveTimeStart(), param.getActiveTimeEnd());
        }

        if (Objects.nonNull(param.getMinCdKeyNum()) && Objects.nonNull(param.getMaxCdKeyNum())) {
            queryWrapper.between("quantity", param.getMinCdKeyNum(), param.getMaxCdKeyNum());
        }

        if (Objects.nonNull(param.getChannelId())) {
            queryWrapper.eq("channel_id", param.getChannelId());
        }

        if (Objects.nonNull(param.getStatus())) {
            queryWrapper.eq("status", param.getStatus());
        }

        queryWrapper.orderByDesc("FIELD(`status`,'waiting','nonactivated','activated','invalid')");
        queryWrapper.orderByDesc("created_time");

        startPage();
        List<CdKeyGroupVO> cdKeyGroupList = cdKeyGroupService.listCdKeyGroupListVO(queryWrapper);
        if (CollectionUtils.isNotEmpty(cdKeyGroupList)) {
            cdKeyGroupList.forEach(cdKeyGroupVO -> {
                        Channel channel = channelService.getById(cdKeyGroupVO.getChannelId());
                        String siteName = channel.getSiteName() == null ? "--   " : channel.getSiteName();
                        cdKeyGroupVO.setSiteName(siteName + "(" + channel.getMobile() + ")");
            });
        }
        return getDataTable(cdKeyGroupList);
    }

    @ApiOperation("兑换码详情")
    @PreAuthorize("@ss.hasPermi('business:cdKey:query')")
    @PostMapping("/detail/{groupId}")
    @ApiResponses(
            @ApiResponse(code = 200, message = "查询成功", response = CdKeyGroupVO.class)
    )
    public AjaxResult detail(@PathVariable String groupId) {
        CdKeyGroup cdKeyGroup = cdKeyGroupService.getById(groupId);
        CdKeyGroupVO cdKeyGroupVO = mapperFacade.map(cdKeyGroup, CdKeyGroupVO.class);
        List<CdKeyGroupGoods> goodsList = cdKeyGroupGoodsService.list(new QueryWrapper<CdKeyGroupGoods>().eq("group_id", groupId));
        cdKeyGroupVO.setGoodsList(goodsList);
        return AjaxResult.success(cdKeyGroupVO);
    }

    @ApiOperation("激活兑换码")
    @PreAuthorize("@ss.hasPermi('business:cdKey:activate')")
    @PostMapping("/activate")
    public AjaxResult activate(@RequestBody CdKeyActivateParam param) {
        CdKeyGroup cdKeyGroup = cdKeyGroupService.getById(param.getGroupId());
        if (!CdKeyGroupStatusEnum.NONACTIVATED.getValue().equals(cdKeyGroup.getStatus().getValue())) {
            return AjaxResult.error("只有未激活的兑换码组才能激活");
        }
        cdKeyGroup.setStatus(param.getIsActive() == 1 ? CdKeyGroupStatusEnum.ACTIVATED : CdKeyGroupStatusEnum.NONACTIVATED);
        cdKeyGroup.setActivationTime(new Date());
        return AjaxResult.success(cdKeyGroupService.updateById(cdKeyGroup));
    }

    @ApiOperation("查看兑换码兑换详情")
    @PreAuthorize("@ss.hasPermi('business:cdKey:query')")
    @PostMapping("/exchangeDetail/{groupId}")
    @ApiResponses(
            @ApiResponse(code = 200, message = "查询成功", response = CdKeyExchangeVO.class)
    )
    public TableDataInfo exchangeDetail(@PathVariable String groupId) {
        startPage();
        List<CdKeyExchangeVO> cdKeyExchangeVOList = cdKeyService.listCdKeyExchangeVO(new QueryWrapper<CdKey>().eq("t1.group_id", groupId).orderByDesc("t2.exchange_time"));
        long unCashedCount = cdKeyExchangeVOList.stream().filter(cdKey -> CdKeyStatusEnum.UNCASHED.getValue().equals(cdKey.getStatus())).count();
        long cashedCount = cdKeyExchangeVOList.stream().filter(cdKey -> CdKeyStatusEnum.CASHED.getValue().equals(cdKey.getStatus())).count();
        cdKeyExchangeVOList.forEach(cdKeyExchangeVO -> {
            cdKeyExchangeVO.setRemainQty((int) unCashedCount);
            cdKeyExchangeVO.setCashedQty((int) cashedCount);
        });
        TableDataInfo resp = getDataTable(cdKeyExchangeVOList);
        resp.setRows(cdKeyExchangeVOList);
        return resp;
    }

    @ApiOperation("导出兑换码订单")
    @PreAuthorize("@ss.hasPermi('business:cdKey:export')")
    @PostMapping("/export")
    public AjaxResult export(@RequestBody CdKeyExportParam param) {
        List<CdKey> cdKeyExportList = cdKeyService.list(new QueryWrapper<CdKey>().in("group_id", param.getGroupIdList()));
        List<CdKeyOrderExcel> excelList = new ArrayList<>();
        cdKeyExportList.forEach(cdKey -> {
            CdKeyOrderExcel cdKeyOrderExcel = new CdKeyOrderExcel();
            cdKeyOrderExcel.setCdKey(cdKey.getCdKey());
            List<CdKeyGroupGoods> goodsList = cdKeyGroupGoodsService.list(new QueryWrapper<CdKeyGroupGoods>().eq("ref_id", cdKey.getRefId()));
            cdKeyOrderExcel.setGoodsName(goodsList.get(0).getTitle());
            String picUrl = "https://" + publicBucketName + ".cos." + region + ".myqcloud.com/" + goodsList.get(0).getPicUrl();
            cdKeyOrderExcel.setPicUrl(picUrl);
            excelList.add(cdKeyOrderExcel);
        });

        ExcelUtil<CdKeyOrderExcel> excelUtil = new ExcelUtil<>(CdKeyOrderExcel.class);
        return excelUtil.exportExcel(excelList, "兑换码订单导出", false);
    }
}
